package com.tumblr.login;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

import com.tumblr.dao.TumblrBlog;
import com.tumblr.dao.TumblrUser;

public class CallbackServlet extends HttpServlet {
    private static final long serialVersionUID = 1L; 
    
    private static JSONObject getJsonObjectFromTumblrApi(String apiURL, OAuthConsumer consumer) {
    	
    	JSONObject json = null;
    	
    	try {
			// create an HTTP request to a protected resource
			URL url = new URL(apiURL);
			HttpURLConnection httpRequest = (HttpURLConnection) url.openConnection();

			// sign the request
			consumer.sign(httpRequest);

			// send the request
			httpRequest.connect();
			
			BufferedReader reader = new BufferedReader(new InputStreamReader(httpRequest.getInputStream(), "UTF-8"));
			StringBuilder builder = new StringBuilder();
			for (String line = null; (line = reader.readLine()) != null;) {
			    builder.append(line).append("\n");
			}
			
			System.out.println(builder.toString());
			json = (JSONObject)new JSONParser().parse(builder.toString());
		} catch (Exception e) {
			e.printStackTrace();
		}
	        	
    	return json;
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    	String verificationCode = null;
        String apiURL = null;
    	String accessToken = null;
    	JSONObject jsonObject = null;
    	JSONObject userObject = null;
    	OAuthConsumer consumer = null;
    	OAuthProvider provider = null;
    	JSONArray blogArray;
    	JSONObject blogMap = null;
    	ArrayList<TumblrBlog> tumblrBlogList = new ArrayList<TumblrBlog>();

        try {
        	
        	HttpSession httpSession = request.getSession();
        	consumer = (OAuthConsumer)httpSession.getAttribute("consumer");
        	provider = (OAuthProvider)httpSession.getAttribute("provider");
            

        	verificationCode = request.getParameter("oauth_verifier");
        	if(null!=verificationCode && null!=consumer && null!=provider) {
        		 provider.retrieveAccessToken(consumer, verificationCode);
        		 accessToken = consumer.getToken();
        	} else {
        		response.sendRedirect(request.getContextPath() + "/error.html");
        		return;
        	}
        	
            if(null!=accessToken) {
            	System.out.println("accessToken: " + accessToken);
            	apiURL = "http://api.tumblr.com/v2/user/info";
            	jsonObject = getJsonObjectFromTumblrApi(apiURL,consumer);
            	
            	userObject = (JSONObject)((JSONObject)jsonObject.get("response")).get("user");
                //System.out.println(userObject.toString());
            	blogArray = (JSONArray)userObject.get("blogs");
                //System.out.println(blogArray.toString());
                
                for(int j=0; j<blogArray.size();j++){
                	
                	blogMap = (JSONObject)blogArray.get(j);
                    //System.out.println(blogMap.toString());
                    tumblrBlogList.add(new TumblrBlog((String)blogMap.get("name"),(String)blogMap.get("url"),((Long)blogMap.get("followers")).toString(),(String)blogMap.get("title"),((Long)blogMap.get("posts")).toString())); 
                }
                
                TumblrUser tumblrUser = new TumblrUser((String)userObject.get("name"),((Long)userObject.get("likes")).toString(),((Long)userObject.get("following")).toString(),tumblrBlogList);
                httpSession.setAttribute("tumblrUser", tumblrUser);
 
            	response.sendRedirect(request.getContextPath() + "/welcome.jsp");
            }
  
            if(null==accessToken)
            	response.sendRedirect(request.getContextPath() + "/error.html");
        	
        } catch (Exception e) {
            response.sendRedirect(request.getContextPath() + "/error.html");
            throw new ServletException(e);
        }
        
    }
}
